package E3DS::BounceParser;
use strict;
use warnings;
use Mail::DeliveryStatus::BounceParser;
use Data::Dumper;
use Email::MIME;
use constant DIRECT_MAIL_BOUNCE_HEADER=>'X-Direct-Mail-Bounce';
use constant DIRECT_MAIL_AR_HEADER=>'X-Direct-Mail-AR';
sub subject_maybe_bounce_notification{my($class,$subject)=@_;
my@regexes=(qr/Mail delivery failed/i,qr/DELIVERY FAILURE/i,qr/Delivery Status Notification/i,qr/Delivery Notification/i,qr/Mail Delivery Problem/i,qr/Message status/i,qr/Returned e?mail/i,qr/Undeliverable Mail/i,qr/Undeliverable:/i,qr/Undelivered Mail Returned to Sender/i,qr/failure notice/i,qr/Delivery Error Report/i,qr/Automatically rejected mail/i,qr/Undeliverable Message/i,qr/Non delivery report/i,qr/non remis/i,qr/Impossibile recapitare/i,qr/Unzustellbar/i,qr/\x{914D}\x{4FE1}\x{4E0D}\x{80FD}/i);
for my $regex(@regexes){return 1 if($subject=~m/$regex/);}return 0;}sub parse{my($class,$opts)=@_;
my$msg_string=$opts->{message_string};
my$log=$opts->{log}||sub{};
my$verbose=$opts->{verbose}||0;
my$bounce=eval{Mail::DeliveryStatus::BounceParser->new($msg_string,{log=>sub{$log->('info',@_)if$verbose;}});};
if($verbose&&($@||!$bounce->is_bounce)){$log->('error',"Error instantiating Mail::DeliveryStatus::BounceParser: $@");}my@results;
if(!$bounce||!$bounce->is_bounce){return@results;}for my $report($bounce->reports){if($report->get('action')&&$report->get('action')!~/failed/){$log->('notice',"Report action is not 'failed': ",$report->get('action'))if$verbose;
next;}my$bounced_email=$report->get('email');
my$std_reason=$report->get('std_reason');
my$reason=$report->get('reason');
$log->('info',"PRE Bounced Email: $bounced_email, Standard Reason: $std_reason, Reason: $reason")if$verbose;
$reason=~s/\s+/ /g;
if($reason=~m/extended inactivity/i){$std_reason="user_unknown";}if($reason=~m/No such person/i){$std_reason="user_unknown";}if($reason=~m/too many hops/i||$reason=~m/loops back/i||$reason=~m/hop count/i||$reason=~m/mail loop/i){$std_reason="mail_loop";}if($reason=~m/x\.co\/rlbounce/i or$reason=~m/too many (messages|recipients)/i){$std_reason="unknown";}if($reason=~m/Unkown user/i){$std_reason="user_unknown";}if($reason=~m/INVALID RECIP/i){$std_reason="user_unknown";}if($reason=~m/user invalid/i){$std_reason="user_unknown";}if($reason=~m/mailbox recipient does not have a mailbox database/i){$std_reason="user_unknown";}if($std_reason eq 'over_quota'){$std_reason=$class->reclassify_over_quota($reason);}if($reason=~m/inexistente/i||$reason=~m/existe tal pessoa/i||$reason=~m/Usuario Desconocido/i||$reason=~m/No existe ese usuario/i||$reason=~m/Nenhuma pessoa neste/i||$reason=~m/existe essa pessoa neste/i){$std_reason="user_unknown";}if($reason=~m/Utilisateur inconnu/i){$std_reason="user_unknown";}if($reason=~m/User unknown/i){$std_reason="user_unknown";}if($reason=~m/Address not present/i){$std_reason="user_unknown";}if($reason=~m/Recipient verify failed/i){$std_reason="user_unknown";}if($reason=~m/User [^\s]+ not found/i){$std_reason="user_unknown";}if($reason=~m/This person no longer works/i){$std_reason="user_unknown";}if($reason=~m/Address refused/i){$std_reason="user_unknown";}if($reason=~m/name service error/i){$std_reason="domain_error";}if($reason=~m/nonexistent domain/i){$std_reason="domain_error";}if($reason=~m/Domain name [^\s]+ does not resolve/i){$std_reason="domain_error";}if($reason=~m/domain is not hosted here/i){$std_reason="domain_error";}if($reason=~m/error writing message: File too large/i){$std_reason='over_quota';}if($reason=~m/Unable to open mailbox/i){$std_reason='over_quota';}if($reason=~m/Too Many Messages in Folder/i){$std_reason='over_quota';}if($reason=~m/Mailbox has exceeded the limit/i){$std_reason='over_quota';}if($reason=~m/Mailbox full/i){$std_reason='over_quota';}if($reason=~m/disk space/i){$std_reason='over_quota';}if($reason=~m/No mailbox here by that name/i){$std_reason="user_unknown";}if($reason=~m/validrcptto/i){$std_reason="user_unknown";}if($reason=~m/no mailbox/i){$std_reason="user_unknown";}if($reason=~m/email account that you tried to reach does not? exists?/i){$std_reason="user_unknown";}if($reason=~m/user does not exist here/i){$std_reason="user_unknown";}if($reason=~m/Mailbox is frozen/i){$std_reason="user_disabled";}if($reason=~m/unavailable mail user/i){$std_reason="user_disabled";}if($reason=~m/AOL will not accept delivery of this message/i){$std_reason="unknown";}if($reason=~m/\b(blacklists?|spam|content|standards|ClamAV)\b/i){$std_reason="spam";}if($reason eq '550 mailbox unavailable - psmtp'){$std_reason="spam";}if($reason=~m/5\.2\.1/&&$reason=~m/\.aol.\com/i){$std_reason="unknown";}if($reason=~m/failed DMARC Evaluation/i){$std_reason="unknown";}$log->('info',"POST Bounced Email: $bounced_email, Standard Reason: $std_reason, Reason: $reason")if$verbose;
my$dm_bounce_header=$class->extract_header(DIRECT_MAIL_BOUNCE_HEADER,$bounce,$msg_string);
my$delivery_id="";
if($dm_bounce_header){$log->('info',DIRECT_MAIL_BOUNCE_HEADER," is \"$dm_bounce_header\"")if$verbose;
if($dm_bounce_header=~m/([^;\s]+); ([^;\s]+); R$/){$delivery_id=$1;
$bounced_email=$2;
$bounced_email=~tr/n-za-mN-ZA-M/a-zA-Z/;}elsif($dm_bounce_header=~m/([^;\s]+); R$/){$bounced_email=$1;
$bounced_email=~tr/n-za-mN-ZA-M/a-zA-Z/;}elsif($dm_bounce_header eq '__rpseed'){$log->('notice',"Bounce from Return Path seed address $bounced_email");}elsif($dm_bounce_header eq '__cc'){$log->('notice',"Bounce from CC address $bounced_email");}else{$log->('notice',"Couldn't parse bounce header: \"$dm_bounce_header\" from message:\n$msg_string");}}else{$log->('notice',DIRECT_MAIL_BOUNCE_HEADER," was not found in message:\n$msg_string");}my$report_email=$report->get('email');
if(lc($report_email=~s/\@/=40/r)eq lc($bounced_email)){$bounced_email=$report_email;}my$dm_ar_header=$class->extract_header(DIRECT_MAIL_AR_HEADER,$bounce,$msg_string);
my$address_history_id="";
$log->('info',DIRECT_MAIL_AR_HEADER," is $dm_ar_header")if$dm_ar_header&&$verbose;
if($dm_ar_header&&$dm_ar_header=~m/^v1; ([a-z0-9_-]+)/i){$address_history_id=$1;}my$results_ref={Reason=>$reason,StandardReason=>$std_reason,FullText=>$msg_string,Relay=>$report->get('host'),};
$results_ref->{Email}=$bounced_email if$bounced_email;
$results_ref->{DeliveryID}=$delivery_id if$delivery_id;
$results_ref->{AddressHistoryID}=$address_history_id if$address_history_id;
$log->('info','Final parsing results: ',Dumper($results_ref))if$verbose;
push@results,$results_ref;}return@results;}sub reclassify_over_quota{my$class=shift;
local$_=shift;
return"over_quota" if(/mailbox\b.*\bfull/ or/storage/i          or/quota/i            or/\s552\s/           or/\s#?5\.2\.2\s/     or/User\s+mailbox\s+exceeds\s+allowed\s+size/i or/Mailbox\s+size\s+limit\s+exceeded/i or/max\s+message\s+size\s+exceeded/i or/Benutzer\s+hat\s+zuviele\s+Mails\s+auf\s+dem\s+Server/i or/exceeded\s+its\s+disk\s+space\s+limit/i);
return"unknown";}sub match_raw_header{my($class,$header,$raw_msg_string)=@_;
if($raw_msg_string=~/^$header: (.+?)\n[^ \t]/ms){my$value=$1;
$value=~s/\s+/ /g;
return$value;}return undef;}sub extract_header{my($class,$field,$bounce,$msg_string)=@_;
my$header;
if($bounce->orig_message&&$bounce->orig_message->head){$header=$bounce->orig_message->head->get($field);}if(!$header&&$bounce->orig_header){$header=$bounce->orig_header->get($field);}if(!$header&&$msg_string){my$mime=Email::MIME->new($msg_string);
for my $part($mime->subparts){if($part->content_type&&$part->content_type eq 'message/global'){$header=Email::MIME->new($part->body)->header($field);
last;}}}if(!$header&&$bounce->orig_text){$header=$class->match_raw_header($field,$bounce->orig_text);}if(!$header&&$msg_string){$header=$class->match_raw_header($field,$msg_string);}$header=~s/\R//g if$header;
return$header;}1;
